<?php
//    Copyright 2013 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

    ini_set("include_path", "./interfaces:./modules:./lang:./conf");

    define('NO_GET_PERMISSIONS',1);
    define('REST_UPDATOR_ID','1:REST');
    define('SOAP_UPDATOR_ID','1:REST');

    require_once("VOMSAPI.php");

    class VOMSREST extends VOMSAPI {
        protected $format = "plain";
        protected $vo;
        protected $methods_map = array ();


        function __construct ($methods_map, $vo = NULL) {
            $this->methods_map = $methods_map;
            $this->vo = ($vo === NULL ) ? $GLOBALS['vo'] : $vo;
        }

        protected function request_failed ($e_name, $e_descr) {
            if ( $this->format == "plain" ) {
                printf("ERROR %s: %s",$e_name, $e_descr);
            }  else if ( $this->format == "json" ) {
                echo json_encode(array("error" => array ("name" => $e_name, "description" => $e_descr)));
            }
            exit(0);
        }

        protected function error_authException   () { return "Access denied"; }
        protected function error_userException   () { return "Request processing failed"; }
        protected function error_methodException () { return "Bad request"; }

        private function implode_r ($glue, $arr) {
            $str = "";
            foreach ($arr as $element) {
                $str .= (is_array($element)) ? implode_r($glue,$a) : "," . $element;
            }
            return $str;
        }

        public function handle ($req) {
            if ( ! isset($req['method']) )
                $this->request_failed("Bad request", "mandatory request variable 'method' is missing");
            $method = $req['method'];
            if ( ! isset($this->methods_map[$method] ) ) 
                $this->request_failed("Server error", sprintf("There is no method %s defined in REST map",$method));
            $params = array ();
            foreach ( $this->methods_map[$method]["paramsorder"] as $param ) {
                if ( isset($req[$param])) {
                    // TODO: check param type according to regex
                    $params[] = $req[$param];
                } else break;
            }
            $result = call_user_func_array(array($this,$method),$params);
            // TODO: result handlers
            return $result;
        }

        public function setformat ($format) {
            $supported_formats = array( 'plain', 'json' );
            if (in_array($format, $supported_formats) ) {
                $this->format = $format;
            } else {
                $this->request_failed("Server error", sprintf("Unsupported response format '%s' is requested",$format));
            }
        }

        public function applyformat ($data) {
            $result = "";
            if ( $this->format == "plain" ) {
                foreach ( $data as $str ) {
                    if ( is_array($str) ) $str = $this->implode_r(":",$str);
                    $result .= $str . "\n";
                }
            } else if ( $this->format == "json" ) {
                $result = json_encode($data);
            }
            return $result;
        }

    }

