<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	define('NO_GET_PERMISSIONS',1);
	define('SOAP_UPDATOR_ID','1:SOAP');

    require_once("VOMSAPI.php");

	// define base handler class
	class VOMSSOAP extends VOMSAPI {
		protected $major_version = 0;
		protected $minor_version = 0;
		protected $patch_version = 0;
		protected $vo;

		function __construct ($wsdl_file, $vo = NULL) {
			$matches = array();
			if ( preg_match('/[^0-9]*([0-9])\.([0-9])\.([0-9])\.wsdl/',$wsdl_file, $matches) ) {
				$this->major_version = $matches[1];
				$this->minor_version = $matches[2];
				$this->patch_version = $matches[3];
				$this->vo = ($vo === NULL ) ? $GLOBALS['vo'] : $vo;
			}
		}

		// errors SOAPFault interface
		protected function request_failed ($e_name, $e_descr) {
			throw new SoapFault($e_name, $e_descr, ob_get_contents(), "");
		}

        protected function error_authException   () { return "Server.authException";    }
        protected function error_userException   () { return "Server.userException";    }
        protected function error_methodException () { return "Server.handlerException"; }

        protected function acl_prefix      () { return "AccessDeniedException: "; }
        protected function syntax_prefix   () { return "VOMSSyntaxException: ";   }
        protected function nogroup_prefix  () { return "NoSuchGroupException: ";  }
        protected function norole_prefix   () { return "NoSuchRoleException: ";   }
        protected function nouser_prefix   () { return "NoSuchUserException: ";   }
        protected function noattribute_prefix () { return "NoSuchAttributeException: "; }
        protected function nomethod_prefix () { return "NoSuchMethodException: "; }

		// WSDL version numbers
		public function getMajorVersionNumber() {
			return $this->major_version;
		}

		public function getMinorVersionNumber() {
			return $this->minor_version;
		}

		public function getPatchVersionNumber() {
			return $this->patch_version;
		}

		// handle overloaded parameters
		protected function getSOAPOverloadedParameter($in0, $names_chain, $method) {
			// handle complex class vs soapenc:string
			if ( is_object($in0) ) {
				// modified non-overloaded WSDL expect object only
				$param1 = is_array($names_chain) ? $names_chain[0] : $names_chain;
				if ( ! isset($in0->$param1) ) {
					// Non-object records without type specified (e.g. <name>class</name>)
					// treated as empty object. In case of not-WSDL clients like original
					// voms-admin-client python tool this is exactly what we have :-(
					// Here is the dirty manual XML parsing hack to get value.
					global $data;
					$matches = array();
					$regex = '@>([^<]+)</[^>]+></([^>:]+:)?'.$method.'>@';
					preg_match($regex,$data,$matches);
					if ( count($matches) ){
						$result = $matches[1];
					} else {
						$this->request_failed ("Server.serverException",
							"ParseException: Failed to process SOAP request.");
					}
				} else {
					// Object passed as expected
					$result = $in0;
					if ( is_array($names_chain) ) {
						foreach ( $names_chain as $param ) {
							$result = $result->$param;
						}
					} else {
						$result = $in0->$names_chain;
					}
				}
			} else {
				// If record type specified (e.g. <name type="xsd:string">class</name>)
				// we can use string directly even if WSDL expect object.
				$result = $in0;
			}
			return $result;
		}

		// get SAOP client request XML data
		public static function getSOAPRequestData() {
			if ($_SERVER["REQUEST_METHOD"] == "POST") {
				// POST SOAP
				$data = file_get_contents('php://input');
			} else {
				// GET request
				// obtain method name
				if ( isset($_GET['method']) ) {
					$method = $_GET['method'];
					$param = next($_GET);
					while (( $param == $GLOBALS['vo'] ) || ( $param == $method ))
					 	$param = next($_GET);
				} else return 0;
				// Construct parameters XML for request
				$req = sprintf('<ns1:%s>',$method);
				$pcnt = 0;
				while ( $param ) {
					$req .= sprintf('<in%d xsi:type="xsd:string">%s</in%d>',$pcnt,$param,$pcnt);
					$pcnt++;
					$param = next($_GET);
				}
				$req .= sprintf('</ns1:%s>',$method);
				// Create SOAP XML 
				// ?'.'> is for vim proper highlighting :-)
				$data = sprintf('<?xml version="1.0" encoding="UTF-8"?'.'>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://glite.org/wsdl/services/org.glite.security.voms.service.admin" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"><SOAP-ENV:Body>%s</SOAP-ENV:Body></SOAP-ENV:Envelope>', $req);
			}
			return $data;
		} 
	}

?>
